//'Copyright 2004 Seapine Software, Inc. All rights reserved.
//
//IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
//in consideration of your agreement to the following terms, and your use, installation,
//modification or redistribution of this Software constitutes acceptance of these terms.
//If you do not agree with these terms, please do not use, install, modify or redistribute
//this Software.
//
//In consideration of your agreement to abide by the following terms, and subject to these
//terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
//this original Seapine software (the Software), to use, reproduce, modify and redistribute
//the Software, with or without modifications, in source and/or binary forms; provided that
//if you redistribute the Software in its entirety and without modifications, you must retain
//this notice and the following text and disclaimers in all such redistributions of the Software.
//Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
//or promote products derived from the Software without specific prior written permission from
//Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
//implied, are granted by Seapine herein, including but not limited to any patent rights that
//may be infringed by your derivative works or by other works in which the Software may be
////incorporated.
//
//The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
//EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
//OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
//
//IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
//REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
//UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
//IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//******************************************************************************

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;

namespace TTPRSSFeed
{
	public class RSSFeed
	{
		private string m_strPath;
		private string m_strDatabase;
		private string m_strUsername;
		private string m_strPassword;
		private CFilter m_filter;
		private DateTime m_dtLastUpdate;

		public RSSFeed(string p_strLocalPath, string p_strDatabase, string p_strUsername, string p_strPassword)
		{
			m_strPath = p_strLocalPath;
			if (!m_strPath.EndsWith("/") && !m_strPath.EndsWith("\\")) { m_strPath += "\\"; }
			m_strDatabase = p_strDatabase;
			m_strUsername = p_strUsername;
			m_strPassword = p_strPassword;
			m_filter = null;
			m_dtLastUpdate = DateTime.Now;
		}

		public bool generateRSSFeed(ttsoapcgi p_cgiEngine, ref long p_lUpdateCnt, ref string p_strError)
		{
			bool fResult = false;
			try
			{
				long lCookie = -1;
				try
				{
					lCookie = p_cgiEngine.DatabaseLogon( m_strDatabase, m_strUsername, m_strPassword);
				}
				catch (Exception exception) { p_strError = exception.Message; }

				if (lCookie > 0)
				{
					//					CTableColumn[] columns = p_cgiEngine.getColumnsForTable( lCookie, "Defect"); // TEMP
					//
					// Let's see if the user followed instructions and created a filter.
					lookForPredefinedFilter(p_cgiEngine, lCookie);

					CTableColumn[] acolumns = null;
					populateDefectSearchColumns(ref acolumns);

					String strFilter = null;
					if (m_filter != null) { strFilter = m_filter.name; }
					CRecordListSoap reclistDefects = p_cgiEngine.getRecordListForTable( lCookie, "Defect", strFilter, acolumns);

					// Set time now since we've already done the fetch to the backend.
					DateTime dtLastUpdate = DateTime.Now;

					if (reclistDefects != null)
					{
						foreach ( CRecordRowSoap recrow in reclistDefects.records)
						{
							// Make sure this is a defect that was created since our last update.
							DateTime dtCreated = DateTime.ParseExact( recrow.row[0].value.ToString(), "M/d/yyyy h:m tt", null);
							// TTP doesn't track seconds, so this event may have 
							// happened after the last update, but if we leave 0 
							// seconds it won't meet the comparison requirements
							dtCreated.AddSeconds(59);

							if (dtCreated.CompareTo(m_dtLastUpdate) >= 0 &&
								dtCreated.Hour >= m_dtLastUpdate.Hour &&
								dtCreated.Minute >= m_dtLastUpdate.Minute)
							{
								try 
								{
									string str = null;
									CDefect defect = p_cgiEngine.getDefect( lCookie, Convert.ToInt32(recrow.row[1].value.ToString()), str, false);
									buildXML(defect, ref p_strError);
									++p_lUpdateCnt;
								}
								catch (Exception exception) { p_strError = exception.Message; }
							}
						}
					}

					fResult = true;
					m_dtLastUpdate = dtLastUpdate;

					p_cgiEngine.DatabaseLogoff(lCookie);
				}
			}
			catch (Exception exception) { p_strError = exception.Message; }

			return (fResult);
		}

		private void lookForPredefinedFilter(ttsoapcgi p_cgiEngine, long p_lCookie)
		{
			try
			{
				CFilter[] afilters = p_cgiEngine.getFilterList(p_lCookie);
				for (int i = 0; i < afilters.Length; ++i)
				{
					if (afilters[i].name.Equals("rssfeed"))
					{
						m_filter = afilters[i];
						break;
					}
				}
			}
			catch (Exception) {}
		}

		private void populateDefectSearchColumns(ref CTableColumn[] p_acolumns)
		{
			long lSize = 2;
			p_acolumns = new CTableColumn[lSize];

			long lIter = 0;
			for (; lIter < lSize; ++lIter)
				p_acolumns[lIter] = new CTableColumn();

			// If you change the ordering of any existing elements, then you'll have to make
			// the appropriate changes elsewhere or stuff won't work as intended.
			lIter = 0;
			p_acolumns[lIter++].name = "Date Created";
			p_acolumns[lIter++].name = "Number";
//			p_acolumns[lIter++].name = "Summary";
//			p_acolumns[lIter++].name = "Date Found";
//			p_acolumns[lIter++].name = "Product";
//			p_acolumns[lIter++].name = "Component";
//			p_acolumns[lIter++].name = "Type";
//			p_acolumns[lIter++].name = "Priority";					// 5
//			p_acolumns[lIter++].name = "Status";
//			p_acolumns[lIter++].name = "Found by";
//			p_acolumns[lIter++].name = "Description";
//			p_acolumns[lIter++].name = "Currently Assigned To";
//			p_acolumns[lIter++].name = "Version Found";				// 10
		}

		protected void buildXML( CDefect p_defect, ref string p_strError)
		{
			string strFilename = m_strPath + m_strDatabase + ".xml";
			XmlDocument xmlDoc = new XmlDocument();

			try
			{
				xmlDoc.Load(strFilename);
			}
			catch(System.IO.FileNotFoundException)
			{	// if file is not found, create a new xml file
				createXmlFile(strFilename, ref p_strError);
			}

			try
			{
				xmlDoc.Load(strFilename);
				XmlNode root = xmlDoc.DocumentElement;

				XmlNode nodeChannel = root.FirstChild;
				if (nodeChannel != null && !doesGUIDExist(nodeChannel, p_defect.defectnumber))
				{
					XmlNode nodeHeadItem = null;
					if (nodeChannel.ChildNodes.Count > 2)
						nodeHeadItem = nodeChannel.FirstChild.NextSibling.NextSibling.NextSibling;

					// If we are over 15 items, then we're gonna chop off the oldest one to make room.
					if (nodeChannel.ChildNodes.Count > 17) // 15 + 2
						nodeChannel.RemoveChild( nodeChannel.LastChild);

					// Build the new Item
					XmlElement nodeNewItem = xmlDoc.CreateElement("item");

					XmlElement nodeTitle = xmlDoc.CreateElement("title");
					XmlText node = xmlDoc.CreateTextNode("");
					node.Value = p_defect.summary;
					nodeTitle.AppendChild(node);

					XmlElement nodeDescription = buildItemDescriptionNode(xmlDoc, p_defect);

					XmlElement nodeGUID = xmlDoc.CreateElement("guid");
					node = xmlDoc.CreateTextNode("");
					node.Value = Convert.ToString(p_defect.defectnumber);
					nodeGUID.AppendChild(node);
					XmlNode attr = xmlDoc.CreateNode(XmlNodeType.Attribute, "isPermaLink", "");
					attr.Value = "false";
					nodeGUID.Attributes.SetNamedItem(attr);

					nodeNewItem.AppendChild( nodeGUID); // Always put first so we know where to look for unique id.
					nodeNewItem.AppendChild( nodeTitle);
					nodeNewItem.AppendChild( nodeDescription);

					// Insert the new item at the front of the list.
					nodeChannel.InsertBefore( nodeNewItem, nodeHeadItem);
				}
				xmlDoc.Save(strFilename);
			}
			catch(System.IO.FileNotFoundException exception) { p_strError = exception.Message; }
			catch (Exception exception) { p_strError = exception.Message; }
		}

		protected XmlElement buildItemDescriptionNode(XmlDocument p_xmlDoc, CDefect p_defect)
		{
			XmlElement nodeDescription = p_xmlDoc.CreateElement("description");

			string strHtml = "";
			strHtml += "<p><b>Description:</b></p>";
			if (p_defect.reportedbylist.Length > 0)
			{
				strHtml += "<p>" + p_defect.reportedbylist[0].comments + "</p>";
				strHtml += "<p></p><p><b>Steps to reproduce:</b></p>";
				strHtml += "<p>" + p_defect.reportedbylist[0].reproducedsteps + "</p>";
			}
			strHtml += "<p></p><p></p>";
			strHtml += "<p><table id=\"data\" cellSpacing=\"0\" cellPadding=\"25\" border=\"0\">";

			// Row 1
//			strHtml += "<tr>";
			strHtml += "<p><b>Status: </b>" + p_defect.state + "</p>";
//			strHtml += "</tr>";

			// Row 2
//			strHtml += "<tr>";
			strHtml += "<p><b>Type: </b>" + p_defect.type + "</p>";
			strHtml += "<p><b>Priority: </b>" + p_defect.priority + "</p>";
//			strHtml += "</tr>";

			// Row 3
//			strHtml += "<tr>";
			strHtml += "<p><b>Product: </b>" + p_defect.product + "</p>";
			strHtml += "<p><b>Component: </b>" + p_defect.component + "</p>";
//			strHtml += "</tr>";

			// Row 4
			if (p_defect.reportedbylist.Length > 0)
			{
//				strHtml += "<tr>";
				strHtml += "<p><b>Found by: </b>" + p_defect.reportedbylist[0].foundby + "</p>";
				strHtml += "<p><b>Date Found: </b>" + p_defect.reportedbylist[0].datefound.ToString("M/d/yyy") + "</p>";
//				strHtml += "</tr>";

				// Row 5
//				strHtml += "<tr>";
				strHtml += "<p><b>Version Found: </b>" + p_defect.reportedbylist[0].foundinversion + "</p>";
//				strHtml += "<td><b>Currently Assigned To: </b>" +  + "</td>";
//				strHtml += "</tr>";
			}

			strHtml += "</table></p>";
			XmlNode node = p_xmlDoc.CreateCDataSection(strHtml);
			nodeDescription.AppendChild( node);
			return (nodeDescription);
		}

		protected void createXmlFile(string p_strFilename, ref string p_strError)
		{
			XmlTextWriter xmlWriter = new XmlTextWriter( p_strFilename, System.Text.Encoding.UTF8);
			xmlWriter.Formatting = Formatting.Indented;
			xmlWriter.WriteProcessingInstruction("xml", "version='1.0' encoding='UTF-8'");
			xmlWriter.WriteStartElement("rss");
			// Do not use WriteEndElement() here
			// it will cause the <Root></Root> to be <Root />
			xmlWriter.Close();

			XmlDocument xmlDoc = new XmlDocument();

			try
			{
				xmlDoc.Load(p_strFilename);
				XmlNode root = xmlDoc.DocumentElement;
				XmlNode attr = xmlDoc.CreateNode(XmlNodeType.Attribute, "version", "");
				attr.Value = "2.0";
				root.Attributes.SetNamedItem(attr);

				XmlElement nodeChannel = xmlDoc.CreateElement("channel");

				XmlElement nodeTitle = xmlDoc.CreateElement("title");
				XmlText node = xmlDoc.CreateTextNode("");
				node.Value = "TestTrack Pro Defects";
				nodeTitle.AppendChild(node);

				XmlElement nodeDescription = xmlDoc.CreateElement("description");
				node = xmlDoc.CreateTextNode("");
				node.Value = "new defects logged in TestTrack Pro";
				nodeDescription.AppendChild(node);

				XmlElement nodeLastBuildDate = xmlDoc.CreateElement("lastBuildDate");
				node = xmlDoc.CreateTextNode("");
				node.Value = DateTime.Now.ToUniversalTime().ToString("r");
				nodeLastBuildDate.AppendChild(node);

				root.AppendChild( nodeChannel);
				nodeChannel.AppendChild( nodeTitle);
				nodeChannel.AppendChild( nodeDescription);
				nodeChannel.AppendChild( nodeLastBuildDate);
				xmlDoc.Save(p_strFilename);
			}
			catch (Exception exception) { p_strError = exception.Message; }
		}

		private bool doesGUIDExist(XmlNode p_nodeParent, long p_lID)
		{
			bool fDoes = false;
			XmlNode node = p_nodeParent.FirstChild.NextSibling.NextSibling.NextSibling;
			for ( ; !fDoes && node != null; node = node.NextSibling)
			{
				fDoes = node.FirstChild.InnerText == Convert.ToString(p_lID);
			}
			return (fDoes);
		}
	}
}